@@ -5,32 +5,15 @@
  GnomeICU
 *********************************/
 
-/*** GnomeICU header files ***/
 #include "common.h"
-
-/*** Toplevel header files ***/
-#include <stdio.h>
-#include <stdlib.h>
-#include <unistd.h>
-#include <netinet/in.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <sys/socket.h>
-#include <arpa/inet.h>
-#include <netdb.h>
-#include <sys/time.h>
-#include <fcntl.h>
-#include <time.h>
-#include <stdarg.h>
-#include <string.h>
-#include <ctype.h>
-#include <assert.h>
-#include <sys/wait.h>
-#include <signal.h>
-#ifdef HAVE_SOCKS5
-#define SOCKS
-#include <socks.h>
-#endif
+#include "chatdlg.h"
+#include "events.h"
+#include "gnomeicu.h"
+#include "history.h"
+#include "packetprint.h"
+#include "rus_conv.h"
+#include "tcp.h"
+#include "util.h"
 
 GtkWidget *chat_filesel;
 
@@ -38,7 +21,6 @@ GSList *Sessions;
 
 void chat_save_got_name( GtkWidget *widget, ChatSession *csession );
 
-void TCPChatHandshake(ChatSession *csession, int lsock, GdkInputCondition cond);
 void TCPChatWaitForNewConnection(ChatContact *ccontact, int sock,
 				GdkInputCondition cond);
 void TCPChatWait3rdPak(ChatContact *ccontact, int sock, GdkInputCondition cond);
@@ -48,409 +30,6 @@ void TCPChatRead(ChatContact *ccontact, int sock, GdkInputCondition cond);
 
 
 /* Global Functions */
-int TCPAcceptChat(int sock, ChatContact *ccontact)
-{
-     BYTE *buffer;
-     unsigned short intsize;
-     unsigned short backport, tempport;
-     /* This must be int, not socklen_t, breaks compilation otherwise */
-     int length = sizeof(struct sockaddr);
-     BYTE *ptr;
-
-     typedef struct {
-	  BYTE uin1[4];
-	  BYTE version[2];
-	  BYTE command[2];
-	  BYTE zero[2];
-	  BYTE uin2[4];
-	  BYTE cmd[2];
-	  BYTE message_length[2];
-     } tcp_head;
-
-     typedef struct {
-	  BYTE ip[4];
-	  BYTE ip_real[4];
-	  BYTE porta[4];
-	  BYTE junk;
-	  BYTE status[2];
-	  BYTE msgcommand[2];
-	  BYTE session_len[2];
-     } tcp_mid;
-
-     typedef struct {
-	  BYTE back_port[2];
-	  BYTE zero[2];
-	  BYTE portb[4];
-	  BYTE seq[4];
-     } tcp_tail;
-
-     tcp_head pack_head;
-     tcp_mid pack_mid;
-     tcp_tail pack_tail;
-
-     struct sockaddr_in my_addr;
-
-     gint cx;
-     gboolean have_tcp_port = FALSE;
-
-#ifdef TRACE_FUNCTION
-     g_print("TCPAcceptChat\n");
-#endif
-
-     if (ccontact->chatsession == NULL)
-     {
-	  ccontact->chatsession = g_malloc0(sizeof(ChatSession));
-	  Sessions = g_slist_append(Sessions, ccontact->chatsession);
-     }
-	 
-
-     if (ccontact->chatsession->listen_socket == 0)
-     {
-	  ccontact->chatsession->listen_socket = socket(AF_INET, SOCK_STREAM, 0);
-	  if (ccontact->chatsession->listen_socket <= 0)
-	       return FALSE;
-	  
-	  for (cx = min_tcp_port; cx <= max_tcp_port; cx++) {
-	       my_addr.sin_family = AF_INET;
-	       my_addr.sin_port = g_htons(cx);
-	       my_addr.sin_addr.s_addr = g_htonl(INADDR_ANY);
-	       memset(&(my_addr.sin_zero), 0x00, 8);
-	       if (bind
-		   (ccontact->chatsession->listen_socket, (struct sockaddr *) &my_addr,
-		    sizeof(struct sockaddr)) != -1)
-		    have_tcp_port = TRUE;
-	       else
-		    continue;
-	       
-	       break;
-	  }
-	  
-	  if (have_tcp_port == FALSE)
-	       return FALSE;
-
-	  listen(ccontact->chatsession->listen_socket, 1);
-	  ccontact->chatsession->gdk_input =
-	       gdk_input_add(ccontact->chatsession->listen_socket, GDK_INPUT_READ,
-			     (GdkInputFunction) TCPChatHandshake, ccontact->chatsession);
-	  
-	  getsockname(ccontact->chatsession->listen_socket, (struct sockaddr *) &my_addr, &length);
-	  
-	  ccontact->chatsession->listen_port = g_ntohs(my_addr.sin_port);
-     }
-     ccontact->chatsession->initiator = (void *) ccontact;
-
-     tempport = ccontact->chatsession->listen_port;
-     backport = (tempport >> 8) + (tempport << 8);
-
-     DW_2_Chars(pack_head.uin1, our_info->uin);
-     Word_2_Chars(pack_head.version, 0x0003);
-     Word_2_Chars(pack_head.command, ICQ_CMDxTCP_ACK);
-     Word_2_Chars(pack_head.zero, 0x0000);
-     DW_2_Chars(pack_head.uin2, our_info->uin);
-     Word_2_Chars(pack_head.cmd, ICQ_CMDxTCP_CHAT);
-     Word_2_Chars(pack_head.message_length, 1);
-
-     DW_2_IP(pack_mid.ip, 0x00000000);
-     DW_2_IP(pack_mid.ip_real, LOCALHOST);
-     DW_2_Chars(pack_mid.porta, our_port);
-     pack_mid.junk = 0x04;
-     Word_2_Chars(pack_mid.status, ICQ_ACKxTCP_ONLINE);
-     Word_2_Chars(pack_mid.msgcommand, 0x0010);
-     Word_2_Chars(pack_mid.session_len, 1);
-
-     Word_2_Chars(pack_tail.back_port, backport);
-     Word_2_Chars(pack_tail.zero, 0x0000);
-     DW_2_Chars(pack_tail.portb, ccontact->chatsession->listen_port);
-     DW_2_Chars(pack_tail.seq, ccontact->seq);
-
-     if (sock != -1) {
-	  intsize =
-	       sizeof(tcp_head) + sizeof(tcp_mid) + sizeof(tcp_tail) + 2;
-	  buffer = (BYTE *) g_malloc0(intsize + 2);
-
-	  Word_2_Chars(buffer, intsize);
-	  ptr = buffer + 2;
-	  memcpy(ptr, &pack_head, sizeof(pack_head));
-	  ptr += sizeof(pack_head);
-	  *ptr = 0x00;
-	  ptr++;
-	  memcpy(ptr, &pack_mid, sizeof(pack_mid));
-	  ptr += sizeof(pack_mid);
-	  *ptr = 0x00;
-	  ptr++;
-	  memcpy(ptr, &pack_tail, sizeof(pack_tail));
-	  ptr += sizeof(pack_tail);
-	  write(sock, buffer, intsize + 2);
-	  packet_print(buffer + 2, intsize,
-		       PACKET_TYPE_TCP | PACKET_DIRECTION_SEND,
-		       "ACCEPT CHAT");
-	  g_free(buffer);
-
-	  if (ccontact->mpartynew > 1)
-	       TCPConnectChat(ccontact->listen_port, ccontact->contact->uin, ccontact->chatsession);
-	  
-     } else
-	  return -1;
-     
-     return 1;
-}
-
-int TCPRefuseChat(int sock, ChatContact *ccontact)
-{
-    BYTE *buffer;
-    unsigned short intsize;
-
-    typedef struct {
-	BYTE uin1[4];
-	BYTE version[2];
-	BYTE command[2];
-	BYTE zero[2];
-	BYTE uin2[4];
-	BYTE cmd[2];
-	BYTE message_length[2];
-    } tcp_head;
-
-    typedef struct {
-	BYTE ip[4];
-	BYTE ip_real[4];
-	BYTE porta[4];
-	BYTE junk;
-	BYTE status[4];
-	BYTE zeroa[4];
-	BYTE zerob[4];
-	BYTE zeroc[2];
-	BYTE zerod;
-	BYTE seq[4];
-    } tcp_tail;
-
-    tcp_head pack_head;
-    tcp_tail pack_tail;
-
-#ifdef TRACE_FUNCTION
-    g_print("TCPRefuseChat\n");
-#endif
-    
-    ccontact->contact = NULL;
-
-    DW_2_Chars(pack_head.uin1, our_info->uin);
-    Word_2_Chars(pack_head.version, 0x0003);
-    Word_2_Chars(pack_head.command, ICQ_CMDxTCP_CANCEL);
-    Word_2_Chars(pack_head.zero, 0x0000);
-    DW_2_Chars(pack_head.uin2, our_info->uin);
-    DW_2_Chars(pack_head.cmd, ICQ_CMDxTCP_CHAT);
-    DW_2_Chars(pack_head.message_length, 1);
-
-    DW_2_IP(pack_tail.ip, our_ip);
-    DW_2_IP(pack_tail.ip_real, LOCALHOST);
-    DW_2_Chars(pack_tail.porta, our_port);
-    pack_tail.junk = 0x04;
-    DW_2_Chars(pack_tail.status, ICQ_ACKxTCP_REFUSE);
-
-    DW_2_Chars(pack_tail.zeroa, 0x00000001);
-    DW_2_Chars(pack_tail.zerob, 0x00000000);
-    DW_2_Chars(pack_tail.zeroc, 0x00000000);
-    pack_tail.zerod = 0x00;
-    DW_2_Chars(pack_tail.seq, ccontact->seq);
-
-    g_free(ccontact);
-
-    if (sock != -1) {
-	intsize = sizeof(tcp_head) + sizeof(tcp_tail) + 1;
-	buffer = (BYTE *) g_malloc0(intsize + 2);
-
-	Word_2_Chars(buffer, intsize);
-	memcpy(buffer + 2, &pack_head, sizeof(pack_head));
-	buffer[2 + sizeof(pack_head)] = 0x00;
-	memcpy(buffer + 2 + sizeof(pack_head) + 1,
-	       &pack_tail, sizeof(pack_tail));
-	write(sock, buffer, intsize + 2);
-	packet_print(buffer + 2, intsize,
-		     PACKET_TYPE_TCP | PACKET_DIRECTION_SEND,
-		     "REFUSE CHAT");
-    } else
-	return -1;
-
-    return 1;
-}
-
-int TCPSendChatRequest(UIN_T uin, gchar * msg, ChatSession *csession)
-{
-    int sock;
-    unsigned short intsize;
-    BYTE *buffer;
-    BYTE *ptr;
-    GSList *temp_ccontact;
-
-    typedef struct {
-	BYTE uin_a[4];
-	BYTE version[2];
-	BYTE cmd[2];
-	BYTE zero[2];
-	BYTE uin_b[4];
-	BYTE command[2];
-	BYTE msg_length[2];
-    } tcp_head;
-
-    typedef struct {
-	BYTE ip[4];
-	BYTE real_ip[4];
-	BYTE port[4];
-	BYTE four;
-	BYTE status[2];
-	BYTE msgcommand[2];
-	BYTE session_len[2];
-    } tcp_mid;
-
-    typedef struct {
-	BYTE revport[2];
-	BYTE zero[2];
-	BYTE port[4];
-	BYTE seq[4];
-    } tcp_tail;
-
-    struct {
-	tcp_head head;
-	const gchar *body;
-	tcp_mid mid;
-	tcp_tail tail;
-    } packet;
-
-    GSList *contact;
-    GString *sessionname = NULL;
-
-#ifdef TRACE_FUNCTION
-    g_print("TCPSendChatRequest\n");
-#endif
-
-    contact = Find_User( uin );
-    if (contact == NULL)
-	return 0;
-
-    
-    if (csession == NULL)
-    {
-	 csession = g_malloc0(sizeof(ChatSession));
-	 Sessions = g_slist_append(Sessions, csession);
-    }
-
-    kontakt->chat_requests = g_list_append(kontakt->chat_requests, csession);
-
-    rus_conv(RUS_KOI_WIN, msg);
-    DW_2_Chars(packet.head.uin_a, our_info->uin);
-    Word_2_Chars(packet.head.version, 0x0003);
-    Word_2_Chars(packet.head.cmd, ICQ_CMDxTCP_START);
-    Word_2_Chars(packet.head.zero, 0x0000);
-    DW_2_Chars(packet.head.uin_b, our_info->uin);
-    Word_2_Chars(packet.head.command, ICQ_CMDxTCP_CHAT);
-    Word_2_Chars(packet.head.msg_length, (strlen(msg) + 1));
-
-    packet.body = msg;
-
-    DW_2_IP(packet.mid.ip, our_ip);
-    DW_2_IP(packet.mid.real_ip, LOCALHOST);
-    DW_2_Chars(packet.mid.port, our_port);
-    packet.mid.four = 0x04;
-
-    switch (Current_Status) {
-    case STATUS_ONLINE:
-	Word_2_Chars(packet.mid.status, ICQ_ACKxTCP_ONLINE);
-	break;
-    case STATUS_AWAY:
-	Word_2_Chars(packet.mid.status, ICQ_ACKxTCP_AWAY);
-	break;
-    case STATUS_DND:
-	Word_2_Chars(packet.mid.status, ICQ_ACKxTCP_DND);
-	break;
-    case STATUS_OCCUPIED:
-	Word_2_Chars(packet.mid.status, ICQ_ACKxTCP_OCC);
-	break;
-    case STATUS_NA:
-	Word_2_Chars(packet.mid.status, ICQ_ACKxTCP_NA);
-	break;
-    case STATUS_INVISIBLE:
-	Word_2_Chars(packet.mid.status, ICQ_ACKxTCP_ONLINE);
-	break;
-    }
-
-    Word_2_Chars(packet.mid.msgcommand, 0x0010);
-
-    if (csession->members && g_slist_length(csession->members) > 0)
-    {
-
-	 sessionname = g_string_new(our_info->nick);
-
-	 for (temp_ccontact = csession->members;
-	      temp_ccontact != NULL;
-	      temp_ccontact = g_slist_next(temp_ccontact))
-	 {
-	      g_string_sprintfa(sessionname, ",%s", ((ChatContact *)temp_ccontact->data)->contact->nick);
-	 }
-	 g_assert (sessionname != NULL);
-	 Word_2_Chars(packet.mid.session_len, sessionname->len);
-	 Word_2_Chars(packet.tail.revport, g_htons((WORD) csession->listen_port));
-	 DW_2_Chars(packet.tail.port, (DWORD) csession->listen_port);
-	 
-
-    }
-    else
-    {
-	 Word_2_Chars(packet.mid.session_len, 1);
-	 Word_2_Chars(packet.tail.revport, 0x0000);
-	 DW_2_Chars(packet.tail.port, 0x00000000);
-    }
-    Word_2_Chars(packet.tail.zero, 0x0000);
-
-    DW_2_Chars(packet.tail.seq, kontakt->tcp_seq);
-
-/*** Create packet ***/
-    intsize =
-	sizeof(tcp_head) + sizeof(tcp_mid) + sizeof(tcp_tail) +
-	strlen(msg) + 1 + ((g_slist_length(csession->members) > 0) ? (sessionname->len): 1 );
-    buffer = (BYTE *) g_malloc0(strlen("REQUEST CHAT") + 1 + intsize + 2);
-
-    Word_2_Chars(buffer, intsize);
-    ptr = buffer + 2;
-    memcpy(ptr, &packet.head, sizeof(packet.head));
-    ptr += sizeof(packet.head);
-    memcpy(ptr, packet.body, strlen(packet.body) + 1);
-    ptr += strlen(packet.body) + 1;
-    memcpy(ptr, &packet.mid, sizeof(packet.mid));
-    ptr += sizeof(packet.mid);
-    if (Chars_2_Word(packet.mid.session_len) == 1)
-    {
-	 *ptr = 0x00;
-	 ptr++;
-    }
-    else
-    {
-	 g_assert(sessionname != NULL);
-	 memcpy(ptr, sessionname->str, sessionname->len);
-	 g_string_free (sessionname, TRUE);
-	 ptr += sessionname->len;
-    }
-    memcpy(ptr, &packet.tail, sizeof(packet.tail));
-    ptr += sizeof(packet.tail);
-    strcpy(buffer + 2 + intsize, "REQUEST CHAT");
-/*********************/
-
-    tcp_prepare_message(kontakt, buffer, NULL,
-			kontakt->tcp_seq--, TRUE);
-
-    sock =
-	TCPGainConnection(kontakt->current_ip,
-			  kontakt->port, contact);
-
-    if (sock == -2)		/* We're waiting for a connection */
-	return TRUE;
-
-    if (sock != -1)
-	TCPClearQueue(contact);
-    else
-	return -1;
-
-    return 1;
-}
-
 int TCPChatSend(GtkWidget *widget, GdkEventKey *ev, ChatSession *csession)
 {
     guchar c;
@@ -665,8 +244,7 @@ void chat_save_got_name(GtkWidget * widget, ChatSession *csession)
     GSList *list;
 
 #ifdef TRACE_FUNCTION
-
-    g_print("chat_save_got_name\n");
+    g_print( "chat_save_got_name" );
 #endif
 
     fn = gtk_file_selection_get_filename(GTK_FILE_SELECTION(csession->filesel));
@@ -743,6 +321,8 @@ int TCPConnectChat( DWORD port, UIN_T uin, ChatSession *csession)
      if (contact == NULL)
 	  return -1;
      
+      if (!kontakt->has_direct_connect)
+          return FALSE;
      
      if (csession == NULL) {
 	  csession = (ChatSession *) g_list_first(kontakt->chat_requests)->data;
@@ -828,7 +408,7 @@ int TCPConnectChat( DWORD port, UIN_T uin, ChatSession *csession)
      getsockname(ccontact->socket, (struct sockaddr *) &local, &sizeofSockaddr);
      ccontact->port = g_ntohs(local.sin_port);
      
-     add_outgoing_to_history(uin, _("Connected for a session."));
+     history_add_outgoing(uin, _("Connected for a session."));
      
      if (csession->chatwindow == NULL)
 	  ChatWindowNew(csession);
@@ -996,19 +576,18 @@ void TCPChatWaitForNewConnection(ChatContact *ccontact, int sock, GdkInputCondit
      GSList *contact;
      WORD font_size = 12;
      gchar *font_name;
-     gchar *font_copy, *font_copy2;
+     gchar *font_copy;
      GSList *otherccontact;
      gchar *message;
  
 #ifdef TRACE_FUNCTION
      g_print("TCPChatWaitForNewConnection\n");
-#endif  
+#endif
      
      gdk_input_remove(ccontact->gdk_input);
 
      font_copy = g_strdup( ChatFontString );
-     font_copy2 = font_copy;
-     strtok( font_copy2, "-" );
+     strtok( font_copy, "-" );
      font_name = strtok( NULL, "-" );
   
      if( read(sock, (char *) (&packet_size), 1) <= 0 )
@@ -1175,7 +754,7 @@ void TCPChatWait3rdPak(ChatContact *ccontact, int sock, GdkInputCondition cond)
      
 #ifdef TRACE_FUNCTION
      g_print("TCPChatWait3rdPak\n");
-#endif  
+#endif
      
      gdk_input_remove(ccontact->gdk_input);
      
@@ -1232,7 +811,7 @@ void TCPChatWaitForAck(ChatContact *ccontact, int sock, GdkInputCondition cond)
      ChatSession *csession = ccontact->chatsession;
      WORD font_size = 12;
      gchar *font_name;
-     gchar *font_copy, *font_copy2;
+     gchar *font_copy;
      GSList *contact;
 
      typedef struct
@@ -1297,8 +876,7 @@ void TCPChatWaitForAck(ChatContact *ccontact, int sock, GdkInputCondition cond)
      g_print("TCPChatWaitForAck\n");
 #endif
      font_copy = g_strdup( ChatFontString );
-     font_copy2 = font_copy;
-     strtok( font_copy2, "-" );
+     strtok( font_copy, "-" );
      font_name = strtok( NULL, "-" );
 
      gdk_input_remove(ccontact->gdk_input);
@@ -1393,10 +971,8 @@ void TCPChatWaitForAck(ChatContact *ccontact, int sock, GdkInputCondition cond)
 		   PACKET_TYPE_TCP | PACKET_DIRECTION_SEND, "CHAT(BEGIN)" );
      g_free( buffer );
 
-     
      g_free( font_copy );
 
-     
      ccontact->gdk_input =
 	  gdk_input_add(ccontact->socket, GDK_INPUT_READ,
 			(GdkInputFunction) TCPChatRead, ccontact); 
