@@ -58,9 +58,7 @@ typedef struct {
 extern GSList *Sessions;
 
 
-int TCPAcceptChat(int sock, ChatContact *ccontact);
-int TCPRefuseChat(int sock, ChatContact *ccontact);
-int TCPSendChatRequest(UIN_T uin, gchar * msg, ChatSession *csession);
+void TCPChatHandshake(ChatSession *csession, int lsock, GdkInputCondition cond);
 int TCPChatSend(GtkWidget * widget, GdkEventKey * ev, ChatSession *csession);
 int TCPChatSendSel(GtkWidget * widget, GtkSelectionData * data, guint time,
 		   ChatSession *csession);
