@@ -4,12 +4,16 @@
  GnomeICU
 ********************************/
 
-/*** GnomeICU header files ***/
 #include "common.h"
+#include "events.h"
+#include "gnomeicu.h"
+#include "util.h"
 
-/*** Toplevel header files ***/
-#include <gdk/gdk.h>
+#include <signal.h>
+#include <sys/wait.h>
 
+void sig_chld( int ); /* prototype of our signalhandler function */
+	
 /*** Global functions ***/
 void gnomeicu_event(ICQEvType event, UIN_T uin)
 {
@@ -40,7 +44,7 @@ void gnomeicu_event(ICQEvType event, UIN_T uin)
 	if( toggles->beep && event != EV_USEROFF )
 		gdk_beep();
 
-#ifdef SOUND
+#ifdef HAVE_ESD
 	if( !toggles->sound )
 		return;
 
@@ -87,11 +91,20 @@ void gnomeicu_fire_external_program( ICQEvType event, CONTACT_PTR contact )
 {
 	char *command=NULL;
 	gchar *arg0, *arg1, *arg2, *arg3, *arg4;
+	static struct sigaction chld;
 
 #ifdef TRACE_FUNCTION
 	g_print( "gnomeicu_fire_external_program(%d,%x)\n", event, (int)contact );
 #endif
 
+	/* set a signalhandler for sigchld to prevente a zombie proccess */
+	if( chld.sa_handler == NULL ){
+		chld.sa_handler = sig_chld;
+		sigemptyset(&chld.sa_mask);
+		chld.sa_flags = SA_NODEFER;
+		sigaction(SIGCHLD, &chld, NULL);
+	}
+	
 	g_return_if_fail ( contact );
 
 	if( !enable_online_events )
@@ -182,7 +195,7 @@ void gnomeicu_fire_external_program( ICQEvType event, CONTACT_PTR contact )
 			arg2 = g_strdup_printf ("%u", contact->uin);
 			arg3 = g_strdup (contact->nick);
 
-			if( contact->current_ip != -1 )
+			if( !contact->has_direct_connect)
 			{
 				arg4 = g_strdup_printf( "%u.%u.%u.%u",
 				       (BYTE) (contact->current_ip >> 24),
@@ -198,3 +211,11 @@ void gnomeicu_fire_external_program( ICQEvType event, CONTACT_PTR contact )
 		}
 	}
 }
+
+/* our signalhandler function for sigchld
+ * nothing to do, just wait for child
+ */
+void sig_chld( int signo ) 
+{
+	while(waitpid(-1,NULL,WNOHANG) > 0);
+}
