@@ -10,7 +10,8 @@
 #include "common.h"
 
 void Read_RC_File( void );
-void Initialize_RC_File( void );
+void save_user_info (void);
+void save_preferences (void);
 void Save_RC( void );
 
 #endif /* __GNOMECFG_H__ */
