@@ -1347,7 +1347,7 @@ void configure_window (GtkWidget *widget, gpointer data)
 	gtk_box_pack_start (GTK_BOX (vbox10), toggle, FALSE, FALSE, 0);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle), temp_toggles->sort_contacts);
 
-	webpresence_button = gtk_check_button_new_with_label (_("Allow others to see me on websites"));
+	webpresence_button = gtk_check_button_new_with_label (_("Allow others to view my status on the ICQ website"));
 	gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (webpresence_button), temp_toggles->webpresence);
 	gtk_widget_show (webpresence_button);
 	gtk_box_pack_start (GTK_BOX (vbox10), webpresence_button, FALSE, FALSE, 0);
