@@ -8,8 +8,10 @@
 #include "gnomeicu.h"
 #include "gtkfunc.h"
 #include "personal_info.h"
+#include "changeinfo.h"
 #include "response.h"
 #include "rus_conv.h"
+#include "kanji_conv.h"
 #include "sendcontact.h"
 #include "showlist.h"
 #include "util.h"
@@ -52,15 +54,15 @@ gchar *v7_search_reply              (DWORD reqid, BYTE result, gchar *here);
 void   v7_end_of_search_reply       (DWORD reqid, BYTE result, gchar *here,
                                      gchar *buffer_end);
 
-void   v7_parse_main_home_info      (DWORD reqid, Contact_Member *DestContact, 
+void   v7_parse_main_home_info      (Request *requestdat, USER_INFO_PTR info, 
                                      BYTE result, gchar *here);
-void   v7_parse_more_info           (DWORD reqid, Contact_Member *DestContact, 
+void   v7_parse_more_info           (Request *requestdat, USER_INFO_PTR info, 
                                      BYTE result, gchar *here);
-void   v7_parse_about_info          (DWORD reqid, Contact_Member *DestContact, 
+void   v7_parse_about_info          (Request *requestdat, USER_INFO_PTR info, 
                                      BYTE result, gchar *here);
-void   v7_parse_work_info           (DWORD reqid, Contact_Member *DestContact, 
+void   v7_parse_work_info           (Request *requestdat, USER_INFO_PTR info, 
                                      BYTE result, gchar *here);
-void   v7_parse_emails_info         (DWORD reqid, Contact_Member *DestContact, 
+void   v7_parse_emails_info         (Request *requestdat, USER_INFO_PTR info, 
                                      BYTE result, gchar *here);
 
 
@@ -167,6 +169,7 @@ gboolean v7_handler (GIOChannel *iochannel, GIOCondition condition,
     
   default:
     g_warning("New unknown channel: %d", packet->channel);
+    break;
   }
 
   g_free(packet->data);
@@ -209,6 +212,7 @@ void v7_snac_channel_handler(V7Packet *packet, V7Connection *conn)
       /* We dont understand the data, ignore it */
       break;
     default:
+      break;
     }
     break;
   case FAMILY_02_LOCATION:
@@ -217,6 +221,7 @@ void v7_snac_channel_handler(V7Packet *packet, V7Connection *conn)
       /* We dont understand the data, ignore it */
       break;
     default:
+      break;
     }
     break;
   case FAMILY_03_CONTACTLIST:
@@ -231,6 +236,7 @@ void v7_snac_channel_handler(V7Packet *packet, V7Connection *conn)
       v7_outgoing_user(snac);
       break;
     default:
+      break;
     }
     break;
   case FAMILY_04_MESSAGING:
@@ -245,6 +251,7 @@ void v7_snac_channel_handler(V7Packet *packet, V7Connection *conn)
       v7_acked_adv_msg(snac, conn);
       break;
     default:
+      break;
     }
     break;
   case FAMILY_09_LISTS:
@@ -253,11 +260,13 @@ void v7_snac_channel_handler(V7Packet *packet, V7Connection *conn)
       /* We dont understand the data, ignore it */
       break;
     default:
+      break;
     }
     break;
   case FAMILY_0B_UNKNOWN_1:
     switch (snac->type) {
     default:
+      break;
     }
     break;
   case FAMILY_13_SAVED_LIST:
@@ -284,7 +293,11 @@ void v7_snac_channel_handler(V7Packet *packet, V7Connection *conn)
     case F13_SERVER_AUTH_REQUEST: /* 0x19 */
       v7_recv_auth_request (snac);
       break;
+    case F13_SERVER_USER_GRANT_AUTH: /* 0x1B */
+      v7_user_grant_auth (snac);
+      break;
     default:
+      break;
     }
     break;
   case FAMILY_15_SAVED_DATA:
@@ -292,11 +305,13 @@ void v7_snac_channel_handler(V7Packet *packet, V7Connection *conn)
     case F15_SERVER_SAVED_DATA:
       v7_saved_data(snac);
     default:
+      break;
     }
     break;
   case FAMILY_17_NEW_USER:
     switch (snac->type) {
     default:
+      break;
    }
     break;
   default:
@@ -331,6 +346,10 @@ void v7_newconn_channel_handler (V7Packet *packet, V7Connection *conn)
     if (! flap_send(conn, CHANNEL_NEWCONN, cookietlv->beg, cookietlv->len))
       conn->status = BROKEN;
 
+    g_free(conn->cookie);
+    conn->cookie = NULL;
+    conn->cookielen = 0;
+
     free_tlvstack(cookietlv);
 
     
@@ -350,19 +369,6 @@ void v7_ack_ack_rate    (V7Connection *conn)
 
   snac_send(conn, v7_ack_ack_rate_array, 10, FAMILY_01_GENERIC,
             F01_CLIENT_ACK_ACK_RATE, NULL, ANY_ID);
-
-  /*v7_sendcontactlist();*/
-  /*
-  if (Current_Status == STATUS_INVISIBLE)
-    v7_sendvisiblelist();
-  else
-    v7_sendinvisiblelist();
-
-  if (preset_status != STATUS_OFFLINE)
-    v7_setstatus(preset_status);
-  else
-    v7_setstatus(STATUS_ONLINE);
-  */
 }
 
 void v7_receive_msg (Snac *snac, V7Connection *conn)
@@ -423,6 +429,7 @@ void v7_receive_msg (Snac *snac, V7Connection *conn)
     break;
   default:
     g_warning("unknown message format: %x\n", msgformat);
+    break;
   }
   
 }
@@ -440,7 +447,20 @@ void v7_text_message(UIN_T uin, gchar *here)
 #endif
   
   msgtlv = new_tlv(here); 
+
+  if (msgtlv->type == 4 && msgtlv->len == 0) {
+    TLV *temptlv;
+    temptlv = new_tlv(here + 4 + msgtlv->len);
+    delete_tlv(msgtlv);
+    msgtlv = temptlv;
+  }
   
+  if (msgtlv->type != 2) {
+    g_warning("TLV is %d, expecting 2\n", msgtlv->type);
+    return;
+  }
+    
+
   /* I dont know why 3, but it does work */
   msglen = CharsBE_2_Word(msgtlv->value+ 6 + msgtlv->value[3]); 
   msglen -= 4;                /* to skip the 4 zeros */
@@ -469,7 +489,7 @@ void v7_special_message(UIN_T uin, gchar *here)
 {
 
   WORD msglen;
-  gchar *msg, *desc, *nick, *first, *last, *email, *tmp=NULL;
+  gchar *msg, *url, *nick, *first, *last, *email, *tmp=NULL;
   TLV *msgtlv;
   BYTE msgtype, msgflags;
   time_t msg_time;
@@ -498,9 +518,10 @@ void v7_special_message(UIN_T uin, gchar *here)
     msg_received(uin, msg, msg_time);
     break;
   case 0x04:
-    desc = strchr( msg, '\xFE' ) + 1;
-    *(desc-1) = '\0';
-    url_received (uin, msg, desc, msg_time);
+    url = strchr( msg, '\xFE' ) + 1;
+    *(url-1) = '\0';
+    rus_conv(RUS_WIN_KOI,msg);
+    url_received (uin, url, msg, msg_time);
     break;
   case 0x13:
     v7_rec_contactlist(uin, msg, msglen, msg_time);
@@ -540,18 +561,19 @@ void v7_special_message(UIN_T uin, gchar *here)
     tmp++;
     rus_conv(RUS_WIN_KOI,email);
 
-    desc = tmp;
+    url = tmp;
     tmp = strchr( tmp, '\xFE' );
     if ( tmp == NULL )
       return; /* Bad Packet */
     *tmp = '\0';
     tmp ++;
-    rus_conv(RUS_WIN_KOI,desc);
+    rus_conv(RUS_WIN_KOI,url);
 
-    authorisation_request(uin, nick, first, last, email, desc, msg_time);
+    authorisation_request(uin, nick, first, last, email, url, msg_time);
     break;
   default:
     g_warning("Special message type not handled, type: %x, text: %s\n", msgtype, msg);
+    break;
   }
 
   g_free(msg);
@@ -563,7 +585,7 @@ void v7_advanced_message(UIN_T uin, gchar *data, int len, gchar *begofmes)
   TLV *maintlv, *insidetlv;
   WORD type1, msglen;
   BYTE msgtype;
-  gchar *msg, *here, *desc, *nick, *first, *last, *email, *tmp=NULL, *dertfed;
+  gchar *msg, *here, *url, *nick, *first, *last, *email, *tmp=NULL, *dertfed;
   time_t msg_time;
   int i;
 
@@ -630,9 +652,10 @@ void v7_advanced_message(UIN_T uin, gchar *data, int len, gchar *begofmes)
     g_free(dertfed);
     break;
   case 0x04: /* URL */
-    desc = strchr( msg, '\xFE' ) + 1;
-    *(desc-1) = '\0';
-    url_received (uin, msg, desc, msg_time);
+    url = strchr( msg, '\xFE' ) + 1;
+    *(url-1) = '\0';
+    rus_conv(RUS_WIN_KOI,msg);
+    url_received (uin, url, msg, msg_time);
     break;
   case 0x0C: /* User Added you */
     user_added_you(uin, msg_time);
@@ -677,7 +700,7 @@ void v7_advanced_message(UIN_T uin, gchar *data, int len, gchar *begofmes)
     tmp++;
     rus_conv(RUS_WIN_KOI,email);
 
-    desc = tmp;
+    url = tmp;
     tmp = strchr( tmp, '\xFE' );
     if ( tmp == NULL ) {
       g_free(msg);
@@ -685,9 +708,9 @@ void v7_advanced_message(UIN_T uin, gchar *data, int len, gchar *begofmes)
     }
     *tmp = '\0';
     tmp ++;
-    rus_conv(RUS_WIN_KOI,desc);
+    rus_conv(RUS_WIN_KOI,url);
 
-    authorisation_request(uin, nick, first, last, email, desc, msg_time);
+    authorisation_request(uin, nick, first, last, email, url, msg_time);
     break;
   case 0xE8:  /* Request for away message */
   case 0xE9:
@@ -731,6 +754,7 @@ void v7_advanced_message_ack (UIN_T uin, guchar *data, gchar *begofmes)
 {
 
   TLVstack *tlvs;
+  unsigned char *str=NULL;
 
 #ifdef TRACE_FUNCTION
   g_print( "v7_advanced_message_ack\n" );
@@ -752,8 +776,14 @@ void v7_advanced_message_ack (UIN_T uin, guchar *data, gchar *begofmes)
        Current_Status == STATUS_NA ||
        Current_Status == STATUS_OCCUPIED ||
        Current_Status == STATUS_DND) {
-    add_nontlv_w_le(tlvs, strlen(Away_Message)+1);
-    add_nontlv(tlvs, Away_Message, strlen(Away_Message)+1);
+    if ( toggles->kanji ) {
+      str = kanji_conv_auto(Away_Message, KANJI_SJIS);
+    } else {
+      str = Away_Message;
+    }
+    add_nontlv_w_le(tlvs, strlen(str)+1);
+    add_nontlv(tlvs, str, strlen(str)+1);
+    if ( str != Away_Message ) free(str);
   }
   else
     add_nontlv_w_le(tlvs, 0);  
@@ -804,6 +834,10 @@ void v7_incoming_user (Snac *snac)
       kontakt->has_direct_connect = Chars_2_DW(tlv->value) ? TRUE : FALSE;
       kontakt->port = CharsBE_2_DW(tlv->value+4);
       kontakt->version = CharsBE_2_Word(tlv->value+9);
+      if (kontakt->version > TCP_VERSION)
+        kontakt->used_version = TCP_VERSION;
+      else
+        kontakt->used_version  = kontakt->version;
       break;
     case 0x0A:
       kontakt->current_ip = IP_2_DW( tlv->value );
@@ -888,7 +922,7 @@ void v7_acked_adv_msg (Snac *snac, V7Connection *conn)
 
   awaymessage = g_strndup(snac->data+11+snac->data[10]+2+26+1+20+1+3+2,
                           Chars_2_Word(snac->data+11+snac->data[10]+2+26+1+20+1+3));
-  
+  kanji_conv_to_locale_s(&awaymessage);
   recv_awaymsg( sender_uin, sender_status, awaymessage);
 
 }
@@ -929,6 +963,7 @@ void v7_saved_data_extension (DWORD reqid, gchar *data, gchar *end)
   BYTE result; 
   GSList *requestdat=NULL, *contact=Contacts;
   gchar *nick = NULL;
+  USER_INFO_PTR info = NULL;
 
 #ifdef TRACE_FUNCTION
   g_print( "v7_saved_data_extension\n" );
@@ -940,6 +975,9 @@ void v7_saved_data_extension (DWORD reqid, gchar *data, gchar *end)
   
   data += 3;
 
+
+  /* We do these only if we have a full info request,
+     see it as a type of "if" with 8 ORed conditions */
   switch (subtype) {
   case 0xC800:
 	nick = data + 2;
@@ -955,14 +993,15 @@ void v7_saved_data_extension (DWORD reqid, gchar *data, gchar *end)
 
     if (requestdat == NULL)
       return;
-    
-    while( contact != NULL ) {
-      if( kontakt->uin == ((Request*)(requestdat->data))->uin )
-        break;
-      contact = contact->next;
+
+    if (((Request*)requestdat->data)->type == OUR_INFO) {
+      info = our_info;
+      break;
     }
+
+    contact = Find_User(((Request*)(requestdat->data))->uin);
     
-    if (contact == NULL)
+    if (contact == NULL )
       return;
 
     /* Anti spam action here */
@@ -976,7 +1015,7 @@ void v7_saved_data_extension (DWORD reqid, gchar *data, gchar *end)
         if (kontakt->confirmed == FALSE) {
           kontakt->confirmed = TRUE;
           if (nick != NULL)
-          	strncpy (kontakt->nick, nick, 19);
+            strncpy (kontakt->nick, nick, 19);
           Show_Quick_Status_lower (UPDATE_ONLINE | UPDATE_OFFLINE | UPDATE_NOTLIST, NULL);
         }
       }
@@ -987,11 +1026,19 @@ void v7_saved_data_extension (DWORD reqid, gchar *data, gchar *end)
     if (result != 0x0A)
       return;
     
-    if( kontakt->info == NULL )
-      kontakt->info = g_malloc0( sizeof(USER_INFO_STRUCT));
-    
+    if( kontakt->info == NULL ) {
+      if (kontakt->uin == our_info->uin)
+	kontakt->info = our_info;
+      else
+	kontakt->info = g_malloc0( sizeof(USER_INFO_STRUCT));
+    }
+
+    kontakt->info->uin = kontakt->uin;
+    info = kontakt->info;
+
     break;
   default:
+    break;
   }
 
   switch (subtype) {
@@ -1004,19 +1051,19 @@ void v7_saved_data_extension (DWORD reqid, gchar *data, gchar *end)
     v7_end_of_search_reply(reqid, result, data, end);
     break;
   case 0xC800:
-    v7_parse_main_home_info(reqid, kontakt, result, data);
+    v7_parse_main_home_info(requestdat->data, info, result, data);
     break;
   case 0xDC00:
-    v7_parse_more_info(reqid, kontakt, result, data);
+    v7_parse_more_info(requestdat->data, info, result, data);
     break;
   case 0xD200:
-    v7_parse_work_info(reqid, kontakt, result, data);
+    v7_parse_work_info(requestdat->data, info, result, data);
     break;
   case 0xE600:
-    v7_parse_about_info(reqid, kontakt, result, data);
+    v7_parse_about_info(requestdat->data, info, result, data);
     break;
   case 0xEB00:
-    v7_parse_emails_info(reqid, kontakt, result, data);
+    v7_parse_emails_info(requestdat->data, info, result, data);
   case 0x0E01:
     /* unknown */
   case 0xF000:
@@ -1025,8 +1072,10 @@ void v7_saved_data_extension (DWORD reqid, gchar *data, gchar *end)
     /* past-background-info */
 
     /* This one seems to come last.. so we delete the requestdat here */
-    if (requestdat)
+    if (requestdat) {      
+      g_free(requestdat->data);
       requests = g_slist_remove(requests, requestdat->data);
+    }
     break;
   }
 
@@ -1041,7 +1090,7 @@ void v7_offline_message(DWORD reqid, gchar *data)
   BYTE msgtype, msg_flags;
   WORD msglen;
   gchar *msg = data+14;
-  gchar *desc, *nick, *first, *last, *email, *tmp=NULL;
+  gchar *url, *nick, *first, *last, *email, *tmp=NULL;
 
 #ifdef TRACE_FUNCTION
   g_print( "v7_offline_message\n" );
@@ -1066,9 +1115,10 @@ void v7_offline_message(DWORD reqid, gchar *data)
     msg_received(uin, msg, msg_time);
     break;
   case 0x04:
-    desc = strchr( msg, '\xFE' ) + 1;
-    *(desc-1) = '\0';
-    url_received (uin, msg, desc, msg_time);
+    url = strchr( msg, '\xFE' ) + 1;
+    *(url-1) = '\0';
+    rus_conv(RUS_WIN_KOI,msg);
+    url_received (uin, url, msg, msg_time);
     break;
   case 0x0C:
     user_added_you(uin, msg_time);
@@ -1105,21 +1155,22 @@ void v7_offline_message(DWORD reqid, gchar *data)
     tmp++;
     rus_conv(RUS_WIN_KOI,email);
 
-    desc = tmp;
+    url = tmp;
     tmp = strchr( tmp, '\xFE' );
     if ( tmp == NULL )
       return; /* Bad Packet */
     *tmp = '\0';
     tmp ++;
-    rus_conv(RUS_WIN_KOI,desc);
+    rus_conv(RUS_WIN_KOI,url);
 
-    authorisation_request(uin, nick, first, last, email, desc, msg_time);
+    authorisation_request(uin, nick, first, last, email, url, msg_time);
     break;
   case 0x13:
     v7_rec_contactlist(uin, msg, msglen, msg_time);
     break;
   default:
     g_warning("Offline message type not handled, type: %x, text: %s\n", msgtype, msg);
+    break;
   }
 
 }
@@ -1223,99 +1274,104 @@ void v7_end_of_search_reply(DWORD reqid, BYTE result, gchar *here,
  * - Zip code, Country
  * - Time zone
  */
-void v7_parse_main_home_info (DWORD reqid, Contact_Member *DestContact, 
+void v7_parse_main_home_info (Request *requestdat, USER_INFO_PTR info, 
                               BYTE result, gchar *here)
 {
-  GSList *requestdat;
-
   int len_str;
      
 #ifdef TRACE_FUNCTION
   g_print( "v7_parse_main_home_info\n" );
 #endif 
 
-  requestdat = v7_get_request_by_reqid(reqid);
-  
-  g_free(DestContact->info->nick);
+  g_free(info->nick);
   len_str = Chars_2_Word( here );
   here += 2;
-  DestContact->info->nick = g_strdup( here );
-  rus_conv(RUS_WIN_KOI, DestContact->info->nick);
+  info->nick = g_strdup( here );
+  rus_conv(RUS_WIN_KOI, info->nick);
   here += len_str;
-  if (((Request*)requestdat->data)->type == NEW_USER_VERIFICATION)
-    strncpy(DestContact->nick, DestContact->info->nick, 20);
+  if (requestdat->type == NEW_USER_VERIFICATION) {
+    GSList *contact = Contacts;
+    
+    contact = Find_User( info->uin );
+    
+    if (contact != NULL )
+      strncpy(kontakt->nick, info->nick, 20);
+  }
 
-  g_free(DestContact->info->first);
+  g_free(info->first);
   len_str = Chars_2_Word( here );
   here += 2;
-  DestContact->info->first = g_strdup( here );
-  rus_conv(RUS_WIN_KOI, DestContact->info->first);
+  info->first = g_strdup( here );
+  rus_conv(RUS_WIN_KOI, info->first);
   here += len_str;
 
-  g_free(DestContact->info->last);
+  g_free(info->last);
   len_str = Chars_2_Word( here );
   here += 2;
-  DestContact->info->last = g_strdup( here );
-  rus_conv(RUS_WIN_KOI, DestContact->info->last);
+  info->last = g_strdup( here );
+  rus_conv(RUS_WIN_KOI, info->last);
   here += len_str;
 
-  g_free(DestContact->info->email);
+  g_free(info->email);
   len_str = Chars_2_Word( here );
   here += 2;
-  DestContact->info->email = g_strdup( here );
+  info->email = g_strdup( here );
   here += len_str;
 
-  g_free(DestContact->info->city);
+  g_free(info->city);
   len_str = Chars_2_Word( here );
   here += 2;
-  DestContact->info->city = g_strdup( here );
-  rus_conv(RUS_WIN_KOI, DestContact->info->city);
+  info->city = g_strdup( here );
+  rus_conv(RUS_WIN_KOI, info->city);
   here += len_str;
 
-  g_free(DestContact->info->state);
+  g_free(info->state);
   len_str = Chars_2_Word( here );
   here += 2;
-  DestContact->info->state = g_strdup( here );
-  rus_conv(RUS_WIN_KOI, DestContact->info->state);
+  info->state = g_strdup( here );
+  rus_conv(RUS_WIN_KOI, info->state);
   here += len_str;
 
-  g_free(DestContact->info->phone);
+  g_free(info->phone);
   len_str = Chars_2_Word( here );
   here += 2;
-  DestContact->info->phone = g_strdup( here );
+  info->phone = g_strdup( here );
   here += len_str;
 
-  g_free(DestContact->info->fax);
+  g_free(info->fax);
   len_str = Chars_2_Word( here );
   here += 2;
-  DestContact->info->fax = g_strdup( here );
+  info->fax = g_strdup( here );
   here += len_str;
 
-  g_free(DestContact->info->street);
+  g_free(info->street);
   len_str = Chars_2_Word( here );
   here += 2;
-  DestContact->info->street = g_strdup( here );
-  rus_conv(RUS_WIN_KOI, DestContact->info->street);
+  info->street = g_strdup( here );
+  rus_conv(RUS_WIN_KOI, info->street);
   here += len_str;
 
-  g_free(DestContact->info->cellular);
+  g_free(info->cellular);
   len_str = Chars_2_Word( here );
   here += 2;
-  DestContact->info->cellular = g_strdup( here );
+  info->cellular = g_strdup( here );
   here += len_str;
 
-  g_free(DestContact->info->zip);
+  g_free(info->zip);
   len_str = Chars_2_Word( here );
   here += 2;
-  DestContact->info->zip = g_strdup( here );
+  info->zip = g_strdup( here );
   here += len_str;
 
-  DestContact->info->country = Chars_2_Word( here );
+  info->country = Chars_2_Word( here );
   here += 2;
   
-  DestContact->info->timezone = *here;
+  info->timezone = *here;
 
-  update_personal_info(DestContact->uin);
+  if (requestdat->type == OUR_INFO)
+    update_our_info();
+  else
+    update_personal_info(info->uin);
 }
 
 /* Retrieves:
@@ -1325,7 +1381,7 @@ void v7_parse_main_home_info (DWORD reqid, Contact_Member *DestContact,
  * - Birth year, month, day
  * - Languages (3)
  */
-void v7_parse_more_info (DWORD reqid, Contact_Member *DestContact, 
+void v7_parse_more_info (Request *requestdat, USER_INFO_PTR info, 
                          BYTE result, gchar *here)
 {
   int len_str;
@@ -1335,41 +1391,44 @@ void v7_parse_more_info (DWORD reqid, Contact_Member *DestContact,
 #endif 
   
   if( Chars_2_Word( here ) != (WORD) -1 )
-    DestContact->info->age = Chars_2_Word( here );
+    info->age = Chars_2_Word( here );
   else
-    DestContact->info->age = 0;
+    info->age = 0;
   here += 2;
 
   /* 0x01: FEMALE, 0x02: MALE */
-  DestContact->info->sex = *here;
+  info->sex = *here;
   if ( *here > 2 || *here == 0)
-    DestContact->info->sex = NOT_SPECIFIED;
+    info->sex = NOT_SPECIFIED;
   here++;
 
-  g_free(DestContact->info->homepage);
+  g_free(info->homepage);
   len_str = Chars_2_Word( here );
   here += 2;
-  DestContact->info->homepage = g_strdup( here );
+  info->homepage = g_strdup( here );
   here += len_str;
 
   if( ( *(here+1) > 0 ) && ( *(here+1) < 12 ) )
     {
-      DestContact->info->birth_year = Chars_2_Word(here);
-      DestContact->info->birth_month = *(here+2);
-      DestContact->info->birth_day = *(here+3);
+      info->birth_year = Chars_2_Word(here);
+      info->birth_month = *(here+2);
+      info->birth_day = *(here+3);
     }
-  here += 3;
-  DestContact->info->language1 = here[0];
-  DestContact->info->language2 = here[1];
-  DestContact->info->language3 = here[2];
+  here += 4;
+  info->language1 = here[0];
+  info->language2 = here[1];
+  info->language3 = here[2];
   
-  update_personal_info(DestContact->uin);
+  if (requestdat->type == OUR_INFO)
+    update_our_info();
+  else
+    update_personal_info(info->uin);
 }
 
 /* Retrieves:
  * - About
  */
-void v7_parse_about_info (DWORD reqid, Contact_Member *DestContact, 
+void v7_parse_about_info (Request *requestdat, USER_INFO_PTR info, 
                          BYTE result, gchar *here)
 {
   int len_str;
@@ -1378,13 +1437,16 @@ void v7_parse_about_info (DWORD reqid, Contact_Member *DestContact,
   g_print( "v7_parse_about_info\n" );
 #endif 
   
-  g_free(DestContact->info->about);
+  g_free(info->about);
   len_str = Chars_2_Word( here );
-  here += 2;  DestContact->info->about = g_strdup( here );
-  rus_conv(RUS_WIN_KOI, DestContact->info->about);
+  here += 2;  info->about = g_strdup( here );
+  rus_conv(RUS_WIN_KOI, info->about);
 
-  
-  update_personal_info(DestContact->uin);
+
+  if (requestdat->type == OUR_INFO)
+    update_our_info();
+  else
+    update_personal_info(info->uin);
 }
 
 /* Retrieves:
@@ -1396,7 +1458,7 @@ void v7_parse_about_info (DWORD reqid, Contact_Member *DestContact,
  * - Job position
  * - Work homepage
  */
-void v7_parse_work_info (DWORD reqid, Contact_Member *DestContact, 
+void v7_parse_work_info (Request *requestdat, USER_INFO_PTR info, 
                          BYTE result, gchar *here)
 {
   int len_str;
@@ -1405,83 +1467,86 @@ void v7_parse_work_info (DWORD reqid, Contact_Member *DestContact,
   g_print( "v7_parse_work_info\n" );
 #endif 
   
-  g_free(DestContact->info->work_city);
+  g_free(info->work_city);
   len_str = Chars_2_Word( here );
   here += 2;
-  DestContact->info->work_city = g_strdup( here );
-  rus_conv(RUS_WIN_KOI, DestContact->info->work_city);
+  info->work_city = g_strdup( here );
+  rus_conv(RUS_WIN_KOI, info->work_city);
   here += len_str;
 
-  g_free(DestContact->info->work_state);
+  g_free(info->work_state);
   len_str = Chars_2_Word( here );
   here += 2;
-  DestContact->info->work_state = g_strdup( here );
-  rus_conv(RUS_WIN_KOI, DestContact->info->work_state);
+  info->work_state = g_strdup( here );
+  rus_conv(RUS_WIN_KOI, info->work_state);
   here += len_str;
 
-  g_free(DestContact->info->work_phone);
+  g_free(info->work_phone);
   len_str = Chars_2_Word( here );
   here += 2;
-  DestContact->info->work_phone = g_strdup( here );
+  info->work_phone = g_strdup( here );
   here += len_str;
 
-  g_free(DestContact->info->work_fax);
+  g_free(info->work_fax);
   len_str = Chars_2_Word( here );
   here += 2;
-  DestContact->info->work_fax = g_strdup( here );
+  info->work_fax = g_strdup( here );
   here += len_str;
 
-  g_free(DestContact->info->work_address);
+  g_free(info->work_address);
   len_str = Chars_2_Word( here );
   here += 2;
-  DestContact->info->work_address = g_strdup( here );
-  rus_conv(RUS_WIN_KOI, DestContact->info->work_address);
+  info->work_address = g_strdup( here );
+  rus_conv(RUS_WIN_KOI, info->work_address);
   here += len_str;
   
-  g_free(DestContact->info->work_zip);
+  g_free(info->work_zip);
   len_str = Chars_2_Word( here );
   here += 2;
-  DestContact->info->work_zip = g_strdup( here );
-  rus_conv(RUS_WIN_KOI, DestContact->info->work_zip);
+  info->work_zip = g_strdup( here );
+  rus_conv(RUS_WIN_KOI, info->work_zip);
   here += len_str;
 
-  DestContact->info->work_country = Chars_2_Word( here );
+  info->work_country = Chars_2_Word( here );
   here += 2; /* Skip unknown data */
 
-  g_free(DestContact->info->company_name);
+  g_free(info->company_name);
   len_str = Chars_2_Word( here );
   here += 2;
-  DestContact->info->company_name = g_strdup( here );
-  rus_conv(RUS_WIN_KOI, DestContact->info->company_name);
+  info->company_name = g_strdup( here );
+  rus_conv(RUS_WIN_KOI, info->company_name);
   here += len_str;
 
-  g_free(DestContact->info->department);
+  g_free(info->department);
   len_str = Chars_2_Word( here );
   here += 2;
-  DestContact->info->department = g_strdup( here );
-  rus_conv(RUS_WIN_KOI, DestContact->info->department);
+  info->department = g_strdup( here );
+  rus_conv(RUS_WIN_KOI, info->department);
   here += len_str;
 
-  g_free(DestContact->info->job_pos);
+  g_free(info->job_pos);
   len_str = Chars_2_Word( here );
   here += 2;
-  DestContact->info->job_pos = g_strdup( here );
-  rus_conv(RUS_WIN_KOI, DestContact->info->job_pos);
+  info->job_pos = g_strdup( here );
+  rus_conv(RUS_WIN_KOI, info->job_pos);
   here += len_str;
 
   here += 2; /* Skip unknown here */
 
-  g_free(DestContact->info->work_homepage);
+  g_free(info->work_homepage);
   len_str = Chars_2_Word( here );
   here += 2;
-  DestContact->info->work_homepage = g_strdup( here );
+  info->work_homepage = g_strdup( here );
   here += len_str;
-  
-  update_personal_info(DestContact->uin);
+
+  if (requestdat->type == OUR_INFO)
+    update_our_info();
+  else 
+    update_personal_info(info->uin);
 }
 
     
-void v7_parse_emails_info(DWORD reqid, Contact_Member *DestContact, 
+void v7_parse_emails_info(Request *requestdat, USER_INFO_PTR info, 
                           BYTE result, gchar *here)
 {
   int len_str, count, i;
@@ -1503,11 +1568,14 @@ void v7_parse_emails_info(DWORD reqid, Contact_Member *DestContact,
     here += len_str;
     
     rus_conv(RUS_WIN_KOI, email+1);
-    DestContact->info->emails = g_list_append(DestContact->info->emails,
+    info->emails = g_list_append(info->emails,
                                               email);
   }
 
-  update_personal_info(DestContact->uin);
+  if (requestdat->type == OUR_INFO)
+    update_our_info();
+  else
+    update_personal_info(info->uin);
 }
 
 
@@ -1571,10 +1639,10 @@ void  v7_rate_changed (Snac *snac, V7Connection *conn)
   g_print( "v7_rate_changed\n" );
 #endif 
 
-  here = snac->data;
+  here = snac->data +8;  /* Dont ask why there is a +8 */
 
-  code = CharsBE_2_DW(here);
-  here += 4;
+  code = CharsBE_2_Word(here);
+  here += 2;
   
   /*
     1 = Changed
@@ -1582,35 +1650,38 @@ void  v7_rate_changed (Snac *snac, V7Connection *conn)
     3 = Limit
     4 = Limit cleared
   */
-  class = CharsBE_2_DW(here);
+
+  class = CharsBE_2_Word(here);
+  here += 2;
+  
+  window_size = CharsBE_2_DW(here);
   here += 4;
   
-  window_size = CharsBE_2_Word(here);
-  here += 2;
+  clear = CharsBE_2_DW(here);
+  here += 4;
   
-  clear = CharsBE_2_Word(here);
-  here += 2;
+  alert = CharsBE_2_DW(here);
+  here += 4;
   
-  alert = CharsBE_2_Word(here);
-  here += 2;
+  limit = CharsBE_2_DW(here);
+  here += 4;
   
-  limit = CharsBE_2_Word(here);
-  here += 2;
+  disconnect = CharsBE_2_DW(here);
+  here += 4;
   
-  disconnect = CharsBE_2_Word(here);
-  here += 2;
+  currentavg = CharsBE_2_DW(here);
+  here += 4;
   
-  currentavg = CharsBE_2_Word(here);
-  here += 2;
-  
-  maxavg = CharsBE_2_Word(here);
-  here += 2;
-  
-  /*
+  maxavg = CharsBE_2_DW(here);
+  here += 4;
+
   g_print("Code: %d Class: %d\n", code, class);
   g_print("WinSize: %d Clear: %d Alert: %d Limit: %d Discon: %d Avg: %d MaxAvg:%d\n", window_size, clear, alert, limit, disconnect, currentavg, maxavg);
-  */
-  if (class == 3)
-    gnome_warning_dialog(_("You are sending messages too fast, your last message has been dropped, please wait 10 seconds and resend."));
 
+  if (code == 2)
+    gnome_warning_dialog(_("You are sending too fast, please slow down your sending speed."));
+  
+  if (code == 3)
+    gnome_warning_dialog(_("You are sending messages too fast, your last message has been dropped, please wait 10 seconds and resend. If you continue sending messages at that speed, you will be disconnect by the server. Please give it a break."));
+  
 }
