@@ -65,222 +65,6 @@ void v7_sendmsg(UIN_T uin, gchar *text)
   free_tlvstack(mainpak);
 }
 
-/* DEPRECATED by the new Snac13 server side contacts list */
-void v7_sendcontactlist()
-{
-
-  TLVstack *contactlist;
-  GSList *contact;
-  gchar *strtmp;
-  
-#ifdef TRACE_FUNCTION
-  g_print( "v7_sendcontactlist\n" );
-#endif
-  
-  
-  contactlist = new_tlvstack(NULL,0);
-  contact = Contacts;
-
-  while( contact != NULL ) {
-    if( kontakt->uin > 0 ) {
-      strtmp = g_strdup_printf(" %d", kontakt->uin);
-      strtmp[0] = strlen(strtmp) -1 ;
-      add_nontlv(contactlist, strtmp, strtmp[0]+1);
-      g_free(strtmp);
-    }
-    contact = contact->next;
-  }
-  
-  snac_send(mainconnection, contactlist->beg, contactlist->len,
-            FAMILY_03_CONTACTLIST, F03_CLIENT_ADD_TO_CL, NULL, ANY_ID);
-  
-  free_tlvstack(contactlist);
-}
-
-/* need to clean up all reference to this function because
- * v7_try_add_uin() in v7snac13.c is preferred
- */
-void v7_addcontact(UIN_T uin) 
-{
-  gchar *strtmp;
-  
-#ifdef TRACE_FUNCTION
-  g_print( "v7_sendaddcontact\n" );
-#endif
-  
-  
-  strtmp = g_strdup_printf(" %d", uin);
-  strtmp[0] = strlen(strtmp) -1 ;
-  snac_send(mainconnection, strtmp, strtmp[0]+1,
-            FAMILY_03_CONTACTLIST, F03_CLIENT_ADD_TO_CL, NULL, ANY_ID);
-  
-  g_free(strtmp);
-  v7_request_info (uin);
-}
-
-/* same as v7_addcontact() */
-void v7_remcontact(UIN_T uin) 
-{
-  gchar *strtmp;
-  
-#ifdef TRACE_FUNCTION
-  g_print( "v7_sendremcontact\n" );
-#endif
-  
-  
-  strtmp = g_strdup_printf(" %d", uin);
-  strtmp[0] = strlen(strtmp) -1 ;
-  snac_send(mainconnection, strtmp, strtmp[0]+1,
-            FAMILY_03_CONTACTLIST, F03_CLIENT_REMOVE_FROM_CL, NULL, ANY_ID);
-  
-  g_free(strtmp);
-
-}
-
-
-void v7_sendvisiblelist()
-{
-
-  TLVstack *contactlist;
-  GSList *contact;
-  gchar *strtmp;
-  
-#ifdef TRACE_FUNCTION
-  g_print( "v7_sendvisiblelist\n" );
-#endif
-  
-  
-  contactlist = new_tlvstack(NULL,0);
-  contact = Contacts;
-
-  while( contact != NULL ) {
-    if( kontakt->uin > 0  && kontakt->vis_list) {
-      strtmp = g_strdup_printf(" %d", kontakt->uin);
-      strtmp[0] = strlen(strtmp) -1 ;
-      add_nontlv(contactlist, strtmp, strtmp[0]+1);
-      g_free(strtmp);
-    }
-    contact = contact->next;
-  }
-  
-  snac_send(mainconnection, contactlist->beg, contactlist->len,
-            FAMILY_09_LISTS ,F09_CLIENT_ADD_VISIBLE_LIST , NULL, ANY_ID);
-  
-  free_tlvstack(contactlist);
-}
-
-void v7_addvisible(UIN_T uin) 
-{
-  gchar *strtmp;
-  
-#ifdef TRACE_FUNCTION
-  g_print( "v7_sendaddvisible\n" );
-#endif
-
-  if (Current_Status != STATUS_INVISIBLE)
-    return;
-   
-  
-  strtmp = g_strdup_printf(" %d", uin);
-  strtmp[0] = strlen(strtmp) -1 ;
-  snac_send(mainconnection, strtmp, strtmp[0]+1,
-            FAMILY_09_LISTS, F09_CLIENT_ADD_VISIBLE_LIST, NULL, ANY_ID);
-  
-  g_free(strtmp);
-
-}
-
-void v7_remvisible(UIN_T uin) 
-{
-  gchar *strtmp;
-  
-#ifdef TRACE_FUNCTION
-  g_print( "v7_sendremvisible\n" );
-#endif
-  
-  if (Current_Status != STATUS_INVISIBLE)
-    return;
-  
-  strtmp = g_strdup_printf(" %d", uin);
-  strtmp[0] = strlen(strtmp) -1 ;
-  snac_send(mainconnection, strtmp, strtmp[0]+1,
-            FAMILY_09_LISTS, F09_CLIENT_REM_VISIBLE_LIST, NULL, ANY_ID);
-  
-  g_free(strtmp);
-
-}
-
-void v7_sendinvisiblelist()
-{
-
-  TLVstack *contactlist;
-  GSList *contact;
-  gchar *strtmp;
-  
-#ifdef TRACE_FUNCTION
-  g_print( "v7_sendinvisiblelist\n" );
-#endif
-  
-  
-  contactlist = new_tlvstack(NULL,0);
-  contact = Contacts;
-
-  while( contact != NULL ) {
-    if( kontakt->uin > 0  && kontakt->invis_list) {
-      strtmp = g_strdup_printf(" %d", kontakt->uin);
-      strtmp[0] = strlen(strtmp) -1 ;
-      add_nontlv(contactlist, strtmp, strtmp[0]+1);
-      g_free(strtmp);
-    }
-    contact = contact->next;
-  }
-  
-  snac_send(mainconnection, contactlist->beg, contactlist->len,
-            FAMILY_09_LISTS ,F09_CLIENT_ADD_INVISIBLE_LIST , NULL, ANY_ID);
-  
-  free_tlvstack(contactlist);
-}
-
-void v7_addinvisible(UIN_T uin) 
-{
-  gchar *strtmp;
-  
-#ifdef TRACE_FUNCTION
-  g_print( "v7_sendaddinvisible\n" );
-#endif
-  
-  if (Current_Status == STATUS_INVISIBLE)
-    return;
-  
-  strtmp = g_strdup_printf(" %d", uin);
-  strtmp[0] = strlen(strtmp) -1 ;
-  snac_send(mainconnection, strtmp, strtmp[0]+1,
-            FAMILY_09_LISTS, F09_CLIENT_ADD_INVISIBLE_LIST, NULL, ANY_ID);
-  
-  g_free(strtmp);
-
-}
-
-void v7_reminvisible(UIN_T uin) 
-{
-  gchar *strtmp;
-  
-#ifdef TRACE_FUNCTION
-  g_print( "v7_reminvisible\n" );
-#endif
-  
-  if (Current_Status == STATUS_INVISIBLE)
-    return;
-   
-  strtmp = g_strdup_printf(" %d", uin);
-  strtmp[0] = strlen(strtmp) -1 ;
-  snac_send(mainconnection, strtmp, strtmp[0]+1,
-            FAMILY_09_LISTS, F09_CLIENT_REM_INVISIBLE_LIST, NULL, ANY_ID);
-  
-  g_free(strtmp);
-
-}
-
 void v7_setstatus(DWORD newstatus)
 {
 #ifdef TRACE_FUNCTION
@@ -293,10 +77,8 @@ void v7_setstatus(DWORD newstatus)
   /* Send visible/invisible list.. icq2001b style */
   if (newstatus == STATUS_INVISIBLE) {
     v7_send_status_server (FALSE); /* invisible */
-    /* v7_sendvisiblelist(); */ /* not need with 2001b */
   } else {
     v7_send_status_server (TRUE); /* visible */
-    /* v7_sendinvisiblelist(); */
   }
 
   Current_Status = newstatus;
@@ -320,13 +102,13 @@ void v7_setstatus2(DWORD newstatus)
   /*
     Direct connection aint working anyways... so we disable it by setting the
     IP to zero
+  */
     
   DW_2_IP(directconnectioninfo, our_ip);
   DW_2_CharsBE(directconnectioninfo+4, our_port);
-  */
 
   directconnectioninfo[8] = 4;
-  directconnectioninfo[10] = 8; /* protocol version 8 == 2001b */
+  directconnectioninfo[10] = 7; /* protocol v8=2001b v7=2000b*/
   DW_2_CharsBE(directconnectioninfo+15, 0x50);
   DW_2_CharsBE(directconnectioninfo+19, 3);
   
@@ -389,6 +171,9 @@ void v7_quit()
     otherconnections = g_slist_remove(otherconnections, otherconnections->data);
     
   }
+  
+  isconnecting = FALSE;
+
   animate_off();
 
 }
@@ -756,7 +541,7 @@ DWORD v7_request_info (UIN_T uin)
 #ifdef TRACE_FUNCTION
   g_print( "v7_request_info\n" );
 #endif 
-  
+
   DW_2_Chars(buffer, uin);
   
   inforeq = g_malloc0(sizeof(Request));
@@ -773,14 +558,14 @@ DWORD v7_request_info (UIN_T uin)
 
 }
 
-DWORD v7_request_my_info()
+DWORD v7_request_our_info()
 {
 
   Request *inforeq;
   BYTE buffer[4];
   
 #ifdef TRACE_FUNCTION
-  g_print( "v7_request_my_info\n" );
+  g_print( "v7_request_our_info\n" );
 #endif 
   
   DW_2_Chars(buffer, our_info->uin);
@@ -789,7 +574,7 @@ DWORD v7_request_my_info()
   
   requests = g_slist_append(requests ,inforeq);
   
-  inforeq->type = MY_INFO;
+  inforeq->type = OUR_INFO;
   inforeq->uin = our_info->uin;
 
   inforeq->reqid = v7_send_15_02_D007( 0xD004, buffer, 4);
